# -*- coding: utf-8 -*-
"""
jf-ext.Time.DateExt.py
~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

import time
import datetime


DISPLAY_DATE_FORMAT = "%Y年%m月%d日"
DISPLAY_TIME_FORMAT = "%Y年%m月%d日 %H:%M:%S"
DATE_FORMAT = "%Y-%m-%d"
TIME_FORMAT = "%Y-%m-%d %H:%M:%S"


def date_date_to_date_display_string(date):
    """
    >>> 日期转换成显示日期String
    :param {Date} date:
    :return {String}: 日期的String格式(日期)
    """
    return date.strftime(DISPLAY_DATE_FORMAT)


def date_date_to_time_display_string(date):
    """
    >>> 日期转换成显示时间String
    :param {Date} date:
    :return {String}: 日期的String格式(时间)
    """
    return date.strftime(DISPLAY_TIME_FORMAT)


def date_date_to_date_string(date):
    """
    >>> 日期转换成日期String
    :param {Date} date:
    :return {String}: 日期的String格式(日期)
    """
    return date.strftime(DATE_FORMAT)


def date_date_to_time_string(date):
    """
    >>> 日期转换成时间String
    :param {Date} date:
    :return {String}: 日期的String格式(时间)
    """
    return date.strftime(TIME_FORMAT)


def date_now_date():
    """
    >>> 获取当前时间的日期
    :return {date}: the date of now
    """
    now = datetime.datetime.now()
    return now


def date_today_date():
    """
    >>> 获取当天的日期
    :return {date}: the date of today
    """
    today = datetime.date.today()
    return today


def date_today_date_String():
    """
    >>> 获取当天的日期 String
    :return {String}: the date of today (String)
    """
    today = datetime.date.today()
    todayDateStr = today.strftime(DATE_FORMAT)
    return todayDateStr


def date_diff_from_date(date1, date2):
    """
    >>> 获取日期1 与 日期2 相差天数
    :param {String} date1: 2021-09-10
    :param {String} date2: 2021-09-10
    :return {Int}: the diff days of date1 and date2
    """
    first_date = datetime.datetime.strptime(date1, DATE_FORMAT)
    second_date = datetime.datetime.strptime(date2, DATE_FORMAT)
    return (first_date - second_date).days


def date_weeks_of_year_by_string(date_string):
    """
    >>> 获取日期所在第几周
    :param {String} date_string: 日期
    :return {Int}: 第几周
    """
    date_time = time.strptime(date_string, DATE_FORMAT)
    weeks = time.strftime("%W", date_time)
    return weeks


def date_year_and_weeks_of_year_by_string(date_string):
    """
    >>> 获取日期所在第几周, 与 年份
    :param {String} date_string: 日期
    :return {[Int, Int]}: [年份, 第几周]
    """
    date_time = time.strptime(date_string, DATE_FORMAT)
    weeks = int(time.strftime("%W", date_time))
    year = int(time.strftime("%Y", date_time))
    if weeks < 1:
        weeks += 52
        year -= 1
    return [year, weeks]


def date_start_day_of_weeks(year, weeks):
    """
    >>> 获取一年指定周的周一时间
    :return {Date}:
    """
    # 当年第一天
    year_start_string = "{}-01-01".format(year)
    # 格式化为日期格式
    year_start = datetime.datetime.strptime(year_start_string, '%Y-%m-%d')
    # 当年第一天的周信息
    year_start_calendarmsg = year_start.isocalendar()
    # year_start_week = year_start_calendarmsg[1]
    year_start_weekday = year_start_calendarmsg[2]
    year_start_year = year_start_calendarmsg[0]
    if year_start_year < int(year):
        day_delat = (8 - int(year_start_weekday)) + (int(weeks) - 1) * 7
    else:
        day_delat = (8 - int(year_start_weekday)) + (int(weeks) - 2) * 7
    res = (year_start + datetime.timedelta(days=day_delat)).date()
    return res.strftime(DATE_FORMAT)


def date_time_before_today(days):
    """
    >>> 获取 days 天前时间
    :Returns {String}: the time of  n day before today
    """
    # 先获得时间数组格式的日期
    dateDayAgo = (datetime.datetime.now() - datetime.timedelta(days=days))
    # 转换为其他字符串格式
    otherStyleTime = dateDayAgo.strftime(DATE_FORMAT)
    return otherStyleTime


def date_time_after_today(days):
    """
    >>> 获取 days 天后时间
    :Returns {String}: the time of  n day before today
    """
    # 先获得时间数组格式的日期
    dateDayafter = (datetime.datetime.now() + datetime.timedelta(days=days))
    # 转换为其他字符串格式
    otherStyleTime = dateDayafter.strftime(DATE_FORMAT)
    return otherStyleTime


def date_time_before_date(date, days):
    """
    >>> 获取 days 天前时间
    :param {Date} date:
    :Returns {String}: the time of  n day before today
    """
    # 先获得时间数组格式的日期
    dateDayAgo = (date - datetime.timedelta(days=days))
    # 转换为其他字符串格式
    otherStyleTime = dateDayAgo.strftime(DATE_FORMAT)
    return otherStyleTime


def date_time_after_date(date, days):
    """
    >>> 获取 days 天后时间
    :param {Date} date:
    :Returns {String}: the time of  n day before today
    """
    # 先获得时间数组格式的日期
    dateDayafter = (date + datetime.timedelta(days=days))
    # 转换为其他字符串格式
    otherStyleTime = dateDayafter.strftime(DATE_FORMAT)
    return otherStyleTime


def date_yesterday_time():
    """
    >>> 获取昨天的时间
    :return {String}: the time of yesterday
    """
    oneday = datetime.timedelta(days=1)
    today = datetime.date.today()
    yesterday = today - oneday
    return yesterday


def date_this_monday():
    """
    >>> 获取本周周一日期
    :return: {String}: 返回周一的日期
    """
    # today = datetime.datetime.strptime(str(today), "%Y%m%d")
    today = datetime.date.today()
    return datetime.datetime.strftime(today - datetime.timedelta(today.weekday()), DATE_FORMAT)


def date_this_month_start():
    """
    >>> 获取本月一号
    :return {String}: 返回本月1号日期
    """
    now = datetime.date.today()
    this_month_start = datetime.datetime(now.year, now.month, 1)
    return datetime.datetime.strftime(this_month_start, DATE_FORMAT)


def date_today_time_range():
    """
    >>> 获取当天的时间起始与结束
    :return {[String, String]}: [start_time, end_time]
        [the start time, end time of today]
    """
    return date_time_range_by_date(date_today_date())


def date_yesterday_time_range():
    """
    >>> 获取昨日的时间起始与结束
    :return {[String, String]}: [start_time, end_time]
        [the start time of yesterday, end time of today of yesterday]
    """
    return date_time_range_by_date(date_yesterday_time())


def date_time_range_by_date(date):
    """
    >>> 获取时间起始与结束 by date
    :return {[String, String]}: [mintime, maxtime]
        [the start time, end time of date]
    """
    mintime = str(date) + ' ' + '00:00:00'
    maxtime = str(date) + ' ' + '23:59:59'
    return [mintime, maxtime]


def date_time_range_by_date_cin7(date):
    """
    >>> GMT标准时间 -> 新西兰时间 开始, 结束
    :param {String} date: 国际标准时间 (格式: %Y-%m-%d)
    :return {[String, String]}: [start_time, end_time]
    """
    oneday = datetime.timedelta(days=1)
    new_date = datetime.datetime.strptime(date, DATE_FORMAT)
    new_date_yesterday = new_date - oneday
    yesterday_date = new_date_yesterday.strftime(DATE_FORMAT)

    mintime = str(yesterday_date) + ' ' + '12:00:00'
    maxtime = str(date) + ' ' + '12:00:00'
    return [mintime, maxtime]


def date_string_validate(date_text):
    """
    >>> 时间字符串校验
    :param {String} date: (格式: %Y-%m-%d)
    return {Boolean}: 是否校验成功
    """
    try:
        datetime.datetime.strptime(date_text, '%Y-%m-%d')
        return True
    except ValueError:
        return False



# -*- coding: utf-8 -*-
"""
jf-ext.RequestExt.py
~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""


def get_ip(request):
    """
    >>> 获取客户端IP地址

    @params {} request: flask request
    @returns {String}: request customer IP
    """
    headers_list = request.headers.getlist("X-Forwarded-For")
    ip = headers_list[0] if headers_list else request.remote_addr
    return ip


def get_request_data(request):
    """
    >>> 获取请求数据

    @params {} request: flask request
    @returns {json}: 请求数据
    """
    element = None
    if 'GET' == request.method:
        element = request.args
    if 'POST' == request.method:
        # 判断content_type
        content_type = request.headers.get('Content-Type', '')
        if "application/json" in content_type.lower():
            element = request.json
        else:
            element = request.form
    return element



# -*- coding: utf-8 -*-
"""
jf-ext.Time.TimeExt.py
~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

import time
import datetime

DATE_FORMAT = "%Y-%m-%d"
TIME_FORMAT = "%Y-%m-%d %H:%M:%S"


def time_now_timestamp():
    """
    >>> 时间: 当前时间戳
    :return {Int}: 13位时间戳
    """
    millis = int(round(time.time()))
    return millis


def time_now_string():
    """
    >>> 获取当前时间 String
    :return {String}: the time of now (String)
    """
    now = time.time()
    return time_timestamp_to_string(now)


def time_time_to_string(time_data):
    """
    >>> 时间转换 String
    :return {String}: the time (String)
    """
    return time.strftime(TIME_FORMAT, time_data)


def time_time_to_date(time_data):
    """
    >>> 时间型 -> 日期形
    """
    return datetime.datetime.fromtimestamp(time.mktime(time_data))


def time_time_to_timestamp(time_data):
    """
    >>> 时间型 -> 时间戳
    :return {Int}: 13位时间戳
    """
    millis = int(time.mktime(time_data) * 1000)
    return millis


def time_string_to_date(time_string):
    """
    >>> 时间字符串型 -> 日期形
    """
    return time_time_to_date(time_string_to_time(time_string))


def time_string_to_time(time_string):
    """
    >>> 时间字符串型 -> 时间形
    """
    try:
        return time.strptime(time_string, TIME_FORMAT)
    except Exception:
        pass
    try:
        return time.strptime(time_string, DATE_FORMAT)
    except Exception:
        pass


def time_timestamp_to_string(ts):
    """
    >>> 时间: 时间戳转换时间字符串
    :param {Int} ts: 13 / 10 位时间戳
    :return {String}: 时间字符串
    """
    if 13 == len(str(ts)):
        ts = ts / 1000.0
    time_local = time.localtime(ts)
    time_str = time.strftime(TIME_FORMAT, time_local)
    return time_str


def time_second_before_now_ts(second):
    """
    >>> 时间: 几秒前 时间字符串
    :param {Int} second: 之前几秒
    :return {String}: 时间字符串
    """
    return int(datetime.datetime.timestamp((datetime.datetime.now() - datetime.timedelta(seconds=second))) * 1000)


def time_minute_before_now_ts(minute):
    """
    >>> 时间: 几分钟前 时间字符串
    :param {Int} minute: 之前几分钟
    :return {String}: 时间字符串
    """
    return int(datetime.datetime.timestamp((datetime.datetime.now() - datetime.timedelta(minutes=minute))) * 1000)


def time_hour_before_now_ts(hour):
    """
    >>> 时间: 几小时前 时间字符串
    :param {Int} minute: 之前几小时
    :return {String}: 时间字符串
    """
    return int(datetime.datetime.timestamp((datetime.datetime.now() - datetime.timedelta(hours=hour))) * 1000)


def time_second_before_now_string(second):
    """
    >>> 时间: 几秒前 时间字符串
    :param {Int} second: 之前几秒
    :return {String}: 时间字符串
    """
    return (datetime.datetime.now() - datetime.timedelta(seconds=second)).strftime(TIME_FORMAT)


def time_minute_before_now_string(minute):
    """
    >>> 时间: 几分钟前 时间字符串
    :param {Int} minute: 之前几分钟
    :return {String}: 时间字符串
    """
    return (datetime.datetime.now() - datetime.timedelta(seconds=minute)).strftime(TIME_FORMAT)


def time_hour_before_now_string(hour):
    """
    >>> 时间: 几小时前 时间字符串
    :param {Int} minute: 之前几小时
    :return {String}: 时间字符串
    """
    return (datetime.datetime.now() - datetime.timedelta(seconds=hour)).strftime(TIME_FORMAT)



# -*- coding: utf-8 -*-
"""
jfExt.Mgr.mailMgr.py
~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

from flask_mail import Mail, Message

from ..SingletonExt import Singleton


@Singleton
class MailMgr():
    """
    >>> Mail Manager
    """

    def __init__(self):
        self._mail = None

    @property
    def mail(self):
        assert(self._mail)    # ❗️❗️未初始化 Mail Client
        return self._mail

    @mail.setter
    def mail(self, mail):
        if isinstance(mail, Mail):
            self._mail = mail

    def send_email(self, title, body, recipients=[], bcc=[], files=None):
        """
        >>> 发送邮件
        :param {String} title: 邮件标题
        :param {String} body: 邮件内容
        :param {list<String>} recipients: 收件人列表
        :return {Boolean}: 发送结果状态
        """
        try:
            msg = Message(
                title,
                sender="",
                recipients=recipients,
                bcc=bcc
            )
            msg.html = body
            for file in files:
                file_path = file['file_path']
                file_name = file['file_name']
                file_type = file['file_type']
                with open(file_path, 'r') as fp:
                    msg.attach(file_name, file_type, fp.read())
            self.mail.send(msg)
            return True
        except Exception:
            import traceback
            traceback.print_exc()
            return False



# -*- coding: utf-8 -*-
"""
jf-ext.BasicType.JsonExit.py
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

from datetime import datetime, date


def json_serial(obj):
    """JSON serializer for objects not serializable by default json code"""

    if isinstance(obj, (datetime, date)):
        return obj.isoformat()
    raise TypeError("Type %s not serializable" % type(obj))


def json_flatten_extend(original, extend, mapping={}):
    """
    >>> json字典拍平拓展
    """
    if not isinstance(original, dict) or not isinstance(extend, dict) or not isinstance(mapping, dict):
        return original
    for key in extend.keys():
        # mapping 字段
        if key in mapping.keys():
            new_key = mapping[key]
            original[new_key] = extend[key]
            continue
        # 插入非冲突key
        if key not in original.keys():
            original[key] = extend[key]
            continue



# -*- coding: utf-8 -*-
"""
jf-ext.BasicType.DictExt.py
~~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

import json
from jfExt.EncryptExt import generate_md5


def dict_get_and_insert(dic, key, default):
    """
    >>> 字典: 获取字段, 未找到直接插入
    :param {dictionary} dic: 待处理字典
    :param {String} key: 键
    :param {Any} default: 默认插入值
    """
    if not dic.get(key, None):
        dic[key] = default
    return


def dict_flatten(obj):
    """
    >>> 字典拍平
    """
    from jfExt.BasicType.ListExt import list_to_string
    if not isinstance(obj, dict):
        return False
    new_obj = dict()
    for i in obj.keys():
        # 字典类型展开
        if isinstance(obj[i], dict):
            for j in obj[i].keys():
                tmp = list_to_string(obj[i][j])
                new_obj["{}_{}".format(i, j)] = tmp
            continue
        if isinstance(obj[i], list):
            new_obj[i] = list_to_string(obj[i])
            continue
        new_obj[i] = obj[i]
    return new_obj


def dict_gen_md5_by_model(source):
    """
    >>> 字典生成md5 by model对象
    """
    source['md5'] = ''
    source['update_time'] = ''
    return dict_gen_md5(source)


def dict_check_md5_by_model(source, md5):
    """
    >>> 字典检测md5值是否匹配 by model对象
    """
    source['md5'] = ''
    source['update_time'] = ''
    return dict_check_md5(source, md5)


def dict_gen_md5(source):
    """
    >>> 字典生成md5
    :param {Dictionary} source: 数据源
    :return {String}: md5字符串
    """
    if not isinstance(source, dict):
        return None
    source_json = json.dumps(source)
    return generate_md5(source_json)


def dict_check_md5(source, md5):
    """
    >>> 字典检测md5值是否匹配
    :param {Dictionary} source: 数据源
    :param {String} md5: 待检测md5值
    :return {Boolean}: 是否匹配
    """
    if not isinstance(source, dict):
        return False
    source_md5 = dict_gen_md5(source)
    if source_md5 == md5:
        return True
    else:
        return False



# -*- coding: utf-8 -*-
"""
jf-ext.PrintExt.py
~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

from icecream import ic # noqa:


def print_title(title):
    """
    >>> 打印标题行
    :param {String} title:
    """
    print("")
    print("")
    print(get_color_text_by_style('*' * 50, 0))
    print(get_color_text_by_style(title, 0))
    print(get_color_text_by_style('*' * 50, 0))


def print_sub_title(sub_title):
    """
    >>> 打印小标题行
    :param {String} sub_title:
    """
    print(get_color_text_by_style('-' * 25, 1))
    print(get_color_text_by_style(sub_title, 1))
    print(get_color_text_by_style('-' * 25, 1))


def get_color_text_by_style(text, style):
    """
    >>> 提供带颜色的文字 by style
    :param {Integer} style:
        - 0: 红色字体
    :return {String}:
    """
    if style == 0:
        return "\033[31m{}\033[0m".format(text)
    if style == 1:
        return "\033[32m{}\033[0m".format(text)



# -*- coding: utf-8 -*-
"""
jf-ext.BasicType.StringExt.py
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

import random
import difflib


def string_rm_blank(string):
    """
    >>> 替换字符串中空格 -> _
    :param {String} string: 带空格字符串
    :return {String}: 替换后字符串
    """
    return string.replace(" ", "_", 1000)


def string_remove_non_ascii(text):
    """
    >>> 替换字符串中 非ASCII 字符串
    :param {String} text:
    :return {String}:
    """
    return ''.join([i if ord(i) < 128 else " " for i in text])


def string_key_filter(s):
    """
    >>> 过滤wordpress字符中&编码问题
    """
    if isinstance(s, str):
        # s = str(s).replace('&amp;', '&', 100)
        s = str(s).replace('，', ',', 100)
        s = str(s).replace('\\', ',', 100)
    return s


def string_random(str_len):
    """
    随机生成指定长度的字符串
    :param {Int} str_len: 指定长度
    :return {string}:
    """
    res = ''
    for i in range(str_len):
        res += chr(int(random.random() * 100) % 26 + 65)

    return res


def string_similar(s1, s2):
    """
    >>> 字符串相似度计算
    :param {String} s1: 字符串1
    :param {String} s2: 字符串s
    :return {}: 相似度
    """
    return difflib.SequenceMatcher(None, s1, s2).quick_ratio()


def string_omit_middle(str, keep_len=5):
    """
    >>> 字符串省略中间部分
    :param {String} str: 待处理字符串
    :param {Integer} keep_len: 前后保留长度
    :return {String}: 处理后字符串
    """
    # 判断保留长度是否是否大于字符串本身, 并返回
    if keep_len >= (len(str) - 4) / 2:
        return str
    start_part = str[:keep_len]
    end_part = str[-keep_len:]
    res = "{}...{}".format(start_part, end_part)
    return res



# -*- coding: utf-8 -*-
"""
jf-ext.CurrencyExt.py
~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""


def currency_display_by_int(number):
    """
    >>> money: 金额 书面化 (千位 ,)
    :param {Int} number: 金额
    :return {String}: 金额书面化字符串
    """
    tmp = format(number, ',')
    return tmp


def currency_to_int_times_100(x):
    """
    >>> 金额 -> int * 100
    :param {Float} x:
    :returns {Integer}
    """
    if isinstance(x, float) or isinstance(x, int):
        return int(round(x * 100))
    return 0


def currency_int_to_currency_divide_100(x):
    """
    >>> int -> 金额 / 100
    :param {Int} x:
    :returns {Integer}
    """
    if isinstance(x, int):
        return int(round(x)) / 100.0
    return 0



# -*- coding: utf-8 -*-
"""
jfExt.fileExt.py
~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

from icecream import ic # noqa
import os
import time
import hashlib


# pragma mark - Private
# --------------------------------------------------------------------------------
def TimeStampToTime(timestamp):
    '''
    >>> 把时间戳转化为时间: 1479264792 to 2016-11-16 10:53:12
    '''
    timeStruct = time.localtime(timestamp)
    return time.strftime('%Y-%m-%d %H:%M:%S', timeStruct)


# pragma mark - Public
# --------------------------------------------------------------------------------
def file_get_file_size(file_path):
    '''
    >>> 获取文件的大小,结果保留两位小数, 单位为MB
    '''
    file_path = file_path.encode('utf8')
    fsize = os.path.getsize(file_path)
    fsize = fsize / float(1024 * 1024)
    return round(fsize, 2)


def file_get_file_accesstime(file_path):
    '''
    >>> 获取文件的访问时间
    '''
    file_path = file_path.encode('utf8')
    t = os.path.getatime(file_path)
    return TimeStampToTime(t)


def file_get_file_createtime(file_path):
    '''
    >>> 获取文件的创建时间
    '''
    file_path = file_path.encode('utf8')
    t = os.path.getctime(file_path)
    return TimeStampToTime(t)


def file_get_file_modifytime(file_path):
    '''
    >>> 获取文件的修改时间
    '''
    file_path = file_path.encode('utf8')
    t = os.path.getmtime(file_path)
    return TimeStampToTime(t)


def file_get_file_md5(file_path):
    """
    计算文件的md5
    :param file_name:
    :return:
    """
    m = hashlib.md5()       # 创建md5对象
    with open(file_path, 'rb') as fobj:
        while True:
            data = fobj.read(4096)
            if not data:
                break
            m.update(data)  # 更新md5对象
    return m.hexdigest()    # 返回md5对象
